#!/bin/sh
# Copyright start
# Copyright (C) 2008 - 2020 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end

my_exit(){
    i_ret_code=$1

    # Remove call from bash profile
    sed -i '/config-vm.sh$/d' $f_bash_profile > /dev/null 2>&1

    # Remove csadmin from sudoers
    sudo rm -f /etc/sudoers.d/csadmin

    clear

    # Enable back control C
    trap 2

    if [ 0 -eq $i_ret_code ];then
       #pkill -KILL -u ${USER}
       pkill -KILL -u csadmin
   else
       exit $i_ret_code
   fi
}

add_default_rabbitmq_steps(){

    a_msgs=("${a_msgs[@]}" "Generate certificates")
    a_cmds=("${a_cmds[@]}" "sudo timeout $i_default_timeout_seconds sh $f_config_cust_script")
}

reset_db_password(){
    f_config=/opt/cyops/configs/database/db_config.yml
    f_conf_util=/opt/cyops/configs/scripts/confUtil.py

    # Generate password for PG
    i_hkey=`sudo cat $f_hkey_save_file`
    s_passwd_cyberpgsql=$i_hkey
    # Set the newly generated password
    sudo -Hiu postgres psql -U postgres -c "ALTER USER cyberpgsql WITH ENCRYPTED PASSWORD '$s_passwd_cyberpgsql';"

    # Encrypt newly generated password
    s_passwd_cyberpgsql_encrypted=`python3 /opt/cyops/configs/scripts/manage_passwords.py --encrypt $s_passwd_cyberpgsql`

    # Update the db config with newly generated password
    python3 $f_conf_util -f $f_config -k pg_password -v $s_passwd_cyberpgsql_encrypted

    # enable elasticsearch security
    sudo sh /opt/cyops-search/scripts/setup_elastic.sh $s_passwd_cyberpgsql "$s_passwd_cyberpgsql_encrypted" "fresh"

    # update the nodeid/clusterid in das.ini
    sudo sh /opt/cyops-auth/scripts/update_nodeid_in_das_ini.sh

    # Reset the redis password
    sudo sh /opt/cyops-auth/scripts/reset_redis_password.sh

    # Clear API cache
    cd /opt/cyops-api
    sudo -u nginx php app/console cache:clear --env=prod --no-interaction
}

configure_default_ha_cluster()
{
    # Bydefault, make the node as active primary for fresh install
    csadm ha join-cluster --role primary --status active

    # Please note that we are checking ES jdk in 'configure_default_ha_cluster'. This is NOT
    # related to HA but since we want some place to execute below code, added it here.
    #
    if [ ! -d "/usr/share/elasticsearch/jdk" ]; then
        return
    fi
    #
    # Remove the jdk which get ship with elasticsearch RPM. Please note that there is NO harm
    # in removing jdk that comes with elasticsearch RPM. Following line is mentioned on official
    # website of elasticsearch.
    # "The bundled JVM directory may be removed when using your own JVM."
    # https://www.elastic.co/guide/en/elasticsearch/reference/current/setup.html
    #
    rm -rf /usr/share/elasticsearch/jdk
}

install_ansible(){
    s_version_ansible=2.8.2
    f_pip="/opt/cyops-workflow/.env/bin/pip"
    s_extra_index_url="--extra-index-url https://$product_yum_server/prod/connectors/deps/simple/"
    s_ansible_pip_cmd="sudo -u nginx $f_pip install ansible==$s_version_ansible $s_extra_index_url"
    a_ansible_services="celeryd celerybeatd uwsgi"

    sudo timeout $i_default_timeout_seconds $s_ansible_pip_cmd
    if [ $? = 0 ];then
       for sr in $a_ansible_services
       do
           systemctl restart $sr
           if [ $? != 0 ];then
              echo "Failed to restart $sr service"
           else
              echo "Successfully restarted $sr service"
           fi
           sleep 5
       done
    else
       $f_pip list | grep ansible
    fi
}

add_default_steps(){

    a_msgs=("${a_msgs[@]}" "Set up intra-service authentication")
    a_cmds=("${a_cmds[@]}" "sudo timeout $i_default_timeout_seconds csadm certs --app-key")

    a_msgs=("${a_msgs[@]}" "Generate certificates")
    a_cmds=("${a_cmds[@]}" "sudo timeout $i_default_timeout_seconds csadm certs --generate $s_hostname")

    a_msgs=("${a_msgs[@]}" "Generate Device UUID")
    a_cmds=("${a_cmds[@]}" "process_hardware_key")

    a_msgs=("${a_msgs[@]}" "Reset database passwords")
    a_cmds=("${a_cmds[@]}" "reset_db_password")

    a_msgs=("${a_msgs[@]}" "Restart services")
    a_cmds=("${a_cmds[@]}" "sudo timeout $i_default_timeout_seconds csadm services --restart")

    a_msgs=("${a_msgs[@]}" "Configure default HA cluster")
    a_cmds=("${a_cmds[@]}" "configure_default_ha_cluster")

    if [ $i_response_dialog_form -eq 0 ];then
        a_msgs=("${a_msgs[@]}" "Configure proxy")
        a_cmds=("${a_cmds[@]}" "configure_proxy")
    fi

    a_msgs=("${a_msgs[@]}" "Install python libraries")
    a_cmds=("${a_cmds[@]}" "install_ansible")
}

show_one_liner_mbox(){
    s_msg_temp="$1"
    s_msg_temp_len=${#s_msg_temp}
    i_width_dialog_one_liner_mbox=`expr $s_msg_temp_len + 3`
    show_dialog_mbox "$s_title" \
                     "OK" \
                     "$s_msg_temp" \
                     6 \
                     $i_width_dialog_one_liner_mbox
}

show_dialog_passwdbox(){
    s_title_dialog_passwdbox=$1
    s_ok_label_dialog_passwdbox=$2
    s_cancel_label_dialog_passwdbox=$3
    s_msg_dialog_passwdbox=$4
    i_height_dialog_passwdbox=$5
    i_width_dialog_passwdbox=$6
    f_file=$7
    s_input_string=$8
    $f_dialog_cmd --backtitle "$s_backtitle" \
           --title "| $s_title_dialog_passwdbox |" \
           --insecure \
           --ok-label "$s_ok_label_dialog_passwdbox" \
           --cancel-label "$s_cancel_label_dialog_passwdbox" \
           --passwordbox "$s_msg_dialog_passwdbox" \
           $i_height_dialog_passwdbox \
           $i_width_dialog_passwdbox \
           "$s_input_string" 2>$f_file
}

show_dialog_inputbox(){
    s_title_dialog_inputbox=$1
    s_ok_label_dialog_inputbox=$2
    s_cancel_label_dialog_inputbox=$3
    s_msg_dialog_inputbox=$4
    i_height_dialog_inputbox=$5
    i_width_dialog_inputbox=$6
    f_file=$7
    s_input_string=$8
    $f_dialog_cmd --backtitle "$s_backtitle" \
           --title "| $s_title_dialog_inputbox |" \
           --ok-label "$s_ok_label_dialog_inputbox" \
           --cancel-label "$s_cancel_label_dialog_inputbox" \
           --inputbox "$s_msg_dialog_inputbox" \
           $i_height_dialog_inputbox \
           $i_width_dialog_inputbox \
           "$s_input_string" 2>$f_file
}

show_dialog_yesno(){
    s_title_dialog_yesno=$1
    s_yes_label_dialog_yesno=$2
    s_no_label_dialog_yesno=$3
    s_msg_dialog_yesno=$4
    i_height_dialog_yesno=$5
    i_width_dialog_yesno=$6
    $f_dialog_cmd --backtitle "$s_backtitle" \
           --title "| $s_title_dialog_yesno |" \
           --yes-label "$s_yes_label_dialog_yesno" \
           --no-label "$s_no_label_dialog_yesno" \
           --yesno "$s_msg_dialog_yesno" \
           $i_height_dialog_yesno \
           $i_width_dialog_yesno
    return $?
}

show_dialog_mbox(){
    s_title_dialog_mbox=$1
    s_ok_label_dialog_mbox=$2
    s_msg_dialog_mbox=$3
    i_height_dialog_mbox=$4
    i_width_dialog_mbox=$5
    $f_dialog_cmd --backtitle "$s_backtitle" \
           --title "| $s_title_dialog_mbox |" \
           --ok-label "$s_ok_label_dialog_mbox" \
           --msgbox "$s_msg_dialog_mbox" \
           $i_height_dialog_mbox \
           $i_width_dialog_mbox
}

show_dialog_progress(){
    i_perc=$1
    s_title_dialog_progress=$2
    s_msg_dialog_progress=$3
    i_height_dialog_progress=$4
    i_width_dialog_progress=$5
    echo $i_perc | $f_dialog_cmd --backtitle "$s_backtitle" \
                        --title "| $s_title_dialog_progress |" \
                        --gauge "$s_msg_dialog_progress" \
                        $i_height_dialog_progress \
                        $i_width_dialog_progress \
                        0
}

show_dialog_infobox(){
    s_title_dialog_infobox=$1
    s_msg_dialog_infobox=$2
    i_height_dialog_infobox=$3
    i_width_dialog_infobox=$4
    $f_dialog_cmd --backtitle "$s_backtitle" \
           --title "| $s_title_dialog_infobox |" \
           --infobox "$s_msg_dialog_infobox" \
           $i_height_dialog_infobox \
           $i_width_dialog_infobox
}

# Form to take five inputs
show_dialog_form_five(){
    # Multiple input boxes in single form and set the size of form
    # and inputboxes.
    s_dialog_form_title=$1
    s_dialog_form_label_input1=$2
    s_dialog_form_label_input2=$3
    s_dialog_form_label_input3=$4
    s_dialog_form_label_input4=$5
    s_dialog_form_label_input5=$6
    f_dialog_form_output=$7
    s_dialog_form_text=$8
    i_dialog_form_edit_start=14
    i_dialog_form_edit_end=21

    rm -f $f_dialog_form_output
    $f_dialog_cmd   --title "| $s_dialog_form_title |" \
                    --backtitle "$s_backtitle" \
                    --ok-label "Next" \
                    --no-cancel \
                    --stdout \
                    --form "$s_dialog_form_text" \
                    $i_height_dialog \
                    $i_width_dialog \
                    5 \
                    "$s_dialog_form_label_input1 " 1 1 "$s_dialog_form_value_input1" 1 $i_dialog_form_edit_start $i_dialog_form_edit_end 0 \
                    "$s_dialog_form_label_input2 " 2 1 "$s_dialog_form_value_input2" 2 $i_dialog_form_edit_start $i_dialog_form_edit_end 0 \
                    "$s_dialog_form_label_input3 " 3 1 "$s_dialog_form_value_input3" 3 $i_dialog_form_edit_start $i_dialog_form_edit_end 0 \
                    "$s_dialog_form_label_input4 " 4 1 "$s_dialog_form_value_input4" 4 $i_dialog_form_edit_start $i_dialog_form_edit_end 0 \
                    "$s_dialog_form_label_input5 " 5 1 "$s_dialog_form_value_input5" 5 $i_dialog_form_edit_start $i_dialog_form_edit_end 0 \
                    > $f_dialog_form_output
    return $?
}

show_dialog_form(){
    # Multiple input boxes in single form and set the size of form
    # and inputboxes.
    i_height_dialog_form=$i_height_dialog
    i_width_dialog_form=$i_width_dialog
    i_height_inputbox_form=$i_height_inputbox
    i_width_inputbox_form=$i_width_inputbox
    s_title_dialog_form=$1
    s_form_string_dialog_form=$2
    s_inputbox1_dialog_form=$3
    s_inputbox2_dialog_form=$4
    s_inputbox3_dialog_form=$5
    f_tmp_output=$6
    $f_dialog_cmd  --separate-widget $'\n' --title "| $s_title_dialog_form |" \
         --backtitle "$s_backtitle" --ok-label "Ok" --cancel-label "Skip" \
         --stdout --form "$s_form_string_dialog_form" $i_height_dialog_form $i_width_dialog_form 7 \
         "$s_inputbox1_dialog_form " 1 1 "$s_input1_dialog_form" 1 $i_height_inputbox_form $i_width_inputbox_form 0 \
         "$s_inputbox2_dialog_form " 2 1 "$s_input2_dialog_form" 2 $i_height_inputbox_form $i_width_inputbox_form 0 \
         "$s_inputbox3_dialog_form " 3 1 "$s_input3_dialog_form" 3 $i_height_inputbox_form $i_width_inputbox_form 0 \
         ""                           4 1 ""        4 0 0 0 \
         "$s_subtitle_dialog_form "                 5 1 ""        5 0 0 0 \
         "$s_help1_dialog_form"       6 1 ""        6 0 0 0 \
         "$s_help2_dialog_form"       7 1 ""        7 0 0 0 > $f_tmp_output

}

get_hardware_key(){
    set -x
    echo "Obtaining Device UUID"
    sudo csadm license --get-device-uuid > $f_hkey_save_file
    echo "Obtaining Device UUID: Done"
    set +x
}

calculate_dots(){
    s_string=$1
    i_dots=`expr $i_cols - ${#s_string}`
    s_v=$(printf "%-${i_dots}s" ".")
}

process_hardware_key(){
    get_hardware_key >> $f_log 2>&1 &
    i_pid_get_hardware_key=$!

    wait $i_pid_get_hardware_key
    i_ret_code_get_hardware_key=$?

    if [ 1 -eq $i_ret_code_get_hardware_key ];then
        s_status="Failed"
        i_fail_flag=1
    else
        s_status="Done"
    fi
}

show_in_progress(){
    show_dialog_progress $percentage \
                         "$s_title" \
                         "$s_msg_para[In Progress]" \
                         $i_height_dialog_progress \
                         $i_width_dialog_progress
}

run_cmd(){
    calculate_dots "$s_msg[In Progress]"
    echo "Running: $s_cmd" >> $f_log 2>&1
    if [ "X" = "X$s_msg_full" ];then
        s_msg_para="\n  $s_msg${s_v// /.}"
    else
        s_msg_para="  $s_msg_full\n  $s_msg${s_v// /.}"
    fi

    # Run command in background

    $s_cmd >> $f_log 2>&1 &

    i_pid_cmd=$!

    # Show in progress
    show_in_progress

    # Wait for command to finish
    wait $i_pid_cmd
    ret_code=$?

    echo "*********************************************************************" >> $f_log 2>&1
    echo "" >> $f_log 2>&1
    if [ 1 -eq $ret_code ];then
        s_status="Failed"
        i_fail_flag=1
    else
        s_status="Done"
    fi
    calculate_dots "$s_msg[$s_status]"
    if [ "X" = "X$s_msg_full" ];then
        s_msg_full="\n  $s_msg${s_v// /.}[$s_status]"
    else
        s_msg_full="$s_msg_full\n  $s_msg${s_v// /.}[$s_status]"
    fi
    show_dialog_progress $percentage \
                         "$s_title" \
                         " $s_msg_full" \
                         $i_height_dialog_progress \
                         $i_width_dialog_progress
}

get_proxy_values(){
    s_https=$(cat $f_tmp_output | head -1)
    s_http=$(cat $f_tmp_output | head -2 | tail -1)
    s_noproxy=$(cat $f_tmp_output | head -3 | tail -1)
    s_input1_dialog_form=$s_https
    s_input2_dialog_form=$s_http
    s_input3_dialog_form=$s_noproxy

    # Skip the user and password and compare the pattern https://:@ip:port for the special characters
    s_https_str1=`echo $s_https | cut -d'/' -f1`
    s_https_str2=`echo $s_https | cut -d'@' -f2`
    s_https_final_string=$s_https_str1"//"$s_https_str2

    # Skip the user and password and compare the pattern http://:@ip:port for the special characters
    s_http_str1=`echo $s_http | cut -d'/' -f1`
    s_http_str2=`echo $s_http | cut -d'@' -f2`
    s_http_final_string=$s_http_str1"//"$s_http_str2
}

proxy_validation_message(){
    s_msg=$1
    show_one_liner_mbox "$s_msg"
    # After show_one_liner_mbox function s_input1_dialog_form and s_input2_dialog_form fields will blank.
    # So need maitain previous values of s_input1_dialog_form and s_input2_dialog_form fields.
    show_dialog_form "$s_title_form_dialog" "$s_backtitle_form_dialog" "$s_input1" "$s_input2" "$s_input3" "$f_output"
    i_response_dialog_form=$?
}

validation_proxy(){
    # Check the validation for special character.
    s_https_char="['!#,;$%^&*()<>?_+|']"
    s_noproxy_char="['!@#;:$%^&*()<>?_+|/']"

    while true; do
        get_proxy_values
        if [[ $s_https_final_string =~ $s_https_char  ]] || [[ $s_http_final_string =~ $s_https_char ]]; then
            proxy_validation_message "Do not use any special character in https/http except / @ and :"
        elif [[ $s_noproxy =~ $s_noproxy_char ]]; then
            proxy_validation_message "Do not use any special character except comma (,) between two hostnames or an ip address and a hostname."
        else
            break;
        fi
    done

    echo $s_noproxy | tr \, \\n |
    while read line
    do
         match_hostname_ip $line both
    done
}

set_proxy(){

    # Get the https, http and noproxy URL as inputs.
    # Set the example of the user provided inputs
    s_title_form_dialog="Configure Proxy"
    s_backtitle_form_dialog="Configure Proxy"
    s_input1="HTTPS Proxy"
    s_input2="HTTP Proxy"
    s_input3="No Proxy"
    s_subtitle_dialog_form="Examples"
    s_input1_dialog_form=""
    s_input2_dialog_form=""
    s_input3_dialog_form="127.0.0.1,localhost"
    f_output="output.txt"
    s_help1_dialog_form="HTTP[s]   http[s]://user:password@[ip/fqdn]:port"
    s_help2_dialog_form="No Proxy  [ip1/fqdn1],[ip2/fqdn2]"
    show_dialog_form "$s_title_form_dialog" "$s_backtitle_form_dialog" "$s_input1" "$s_input2" "$s_input3" "$f_output"
    i_response_dialog_form=$?

    validation_proxy

    if [ -z $s_https ] && [ -z $s_http ];then
        i_response_dialog_form=1
        return 1
    fi

    if [ ! -z $s_https ];then
        echo $s_https | grep '@'
        if [ $? -eq 0 ];then
            # Check and Split the username, password, ip and port.
            s_protocol=`echo $s_https | awk -F[/:] '{print $1}'| cut -d'@' -f1`
            s_user=`echo $s_https | awk -F[/:] '{print $4}'| cut -d'@' -f1`
            s_password=`echo $s_https | awk -F[/:] '{print $5}'| cut -d'@' -f1`
            s_host=`echo $s_https | awk -F[/:] '{print $5}'| cut -d'@' -f2`
            i_port=`echo $s_https | awk -F[/:] '{print $6}'| cut -d'@' -f1`
            s_https_cmd="csadm network set-https-proxy --protocol $s_protocol --host $s_host --user $s_user --password $s_password --port $i_port"
        else
            # Check and Split the ip and port.
            s_protocol=`echo $s_https | awk -F[/:] '{print $1}'`
            s_host=`echo $s_https | awk -F[/:] '{print $4}'`
            i_port=`echo $s_https | awk -F[/:] '{print $5}'`
            s_https_cmd="csadm network set-https-proxy --protocol $s_protocol --host $s_host --port $i_port"
        fi
        add_to_summary "HTTPS Proxy" "$s_https"
    fi

    if [ ! -z $s_http ];then
        echo $s_http | grep '@'
        if [ $? -eq 0 ];then
            # Check and Split the username, password, ip and port.
            s_protocol=`echo $s_http | awk -F[/:] '{print $1}'| cut -d'@' -f1`
            s_user=`echo $s_http | awk -F[/:] '{print $4}'| cut -d'@' -f1`
            s_password=`echo $s_http | awk -F[/:] '{print $5}'| cut -d'@' -f1`
            s_host=`echo $s_http | awk -F[/:] '{print $5}'| cut -d'@' -f2`
            i_port=`echo $s_http | awk -F[/:] '{print $6}'| cut -d'@' -f1`
            s_http_cmd="csadm network set-http-proxy --protocol $s_protocol --host $s_host --user $s_user --password $s_password --port $i_port"
        else
            # Check and Split the ip and port.
            s_protocol=`echo $s_http | awk -F[/:] '{print $1}'`
            s_host=`echo $s_http | awk -F[/:] '{print $4}'`
            i_port=`echo $s_http | awk -F[/:] '{print $5}'`
            s_http_cmd="csadm network set-http-proxy --protocol $s_protocol --host $s_host --port $i_port"
        fi
        add_to_summary "HTTP Proxy" "$s_http"
    fi
    if [ $i_response_dialog_form -eq 0 ];then
        s_noproxy_cmd="csadm network set-no-proxy --host $s_noproxy"
        add_to_summary "No Proxy List" "$s_noproxy"
    fi
    rm -f $f_tmp_output
}

configure_proxy(){
   # Call the all command of csadm network excluding ipv6.
   f_etc_environment="/etc/environment"
   f_yum="/etc/yum.conf"
   a_proxy="https_proxy http_proxy no_proxy"
   a_file="$f_etc_environment $f_yum"

   # Check the if https, http and no_proxy exist or not in /etc/environment and yum.conf files.
   # If exist then only change the values of https, http and no_proxy.
   # Avoid the duplicate tag of https, http and no_proxy.
   for f_conf in $a_file
   do
       for str_proxy in $a_proxy
       do
           grep $str_proxy $f_conf
           if [ $? -eq 0 ];then
               sed -i "/$str_proxy/d" $f_conf
           fi
       done
   done

   # Please note that we can add below commands in array and iterate over that array.
   # But not added the loop as commands can have double quotes/space etc. which becomes problematic
   # if not handled carefully.
   echo "Running: set_https_command [$s_https_cmd]"
   $s_https_cmd
   if [ $? -ne 0 ]; then
       return 1
   fi

   echo "Running: set_http_command [$s_http_cmd]"
   $s_http_cmd
   if [ $? -ne 0 ]; then
       return 1
   fi
   # For last command of sub-routine, no need to check the ret code as whatever last command returns,
   # that get sent to caller
   echo "Running: set_noproxy_cmd [$s_noproxy_cmd]"
   $s_noproxy_cmd
}

configure_vm(){
    rm -f $f_log

    #i_steps_count=`expr ${#a_msgs[@]}`
    i_steps_count=`expr ${#a_cmds[@]}`
    i_step_size=`expr 100 / $i_steps_count`
    i_max_percent=`expr 100 - $i_step_size`

    s_title=$s_cust_title

    i_counter=0
    i_fail_flag=0
    i_status_ansible=0
    i_msgs_count=${#a_msgs[@]}
    i_height_dialog_progress=$i_height_dialog
    i_width_dialog_progress=$i_width_dialog

    f_pip="/opt/cyops-workflow/.env/bin/pip"
    s_cmd_rb_user_add="csadm mq --add-user"

    for percentage in $(seq 0 $i_step_size $i_max_percent); do
        s_msg=${a_msgs[$i_counter]}
        s_cmd=${a_cmds[$i_counter]}
        run_cmd
        if [ "$s_msg" = "Generate messaging credentials" ];then
            # Hide rabbitmq user password in log file.
            s_rb_user_password=`grep "$s_cmd_rb_user_add" $f_log`

            if [ ! -z "$s_rb_user_password" ];then
                 # Get the last field of the string.
                 s_tmp_rb_user_password=`echo $s_rb_user_password | awk '{print $NF}'`
                 # Replace the password with **** in s_tmp_rb_user_password variable.
                 s_tmp_rb_cmd=`(echo $s_rb_user_password | sed  "s/$s_tmp_rb_user_password/'****'/g")`
                 # Replace the complete the string with new string s_tmp_rb_cmd in variable.
                 sed -i "s/$s_rb_user_password/'$s_tmp_rb_cmd'/g" $f_log
            fi
        fi

        if [ "$s_cmd" = "install_ansible" ];then
            $f_pip list | grep ansible
            if [ $? != 0 ];then
                i_status_ansible=1
            fi
        fi
        i_counter=`expr $i_counter + 1`
    done

    show_dialog_progress 100 \
                         "$s_title" \
                         " $s_msg_full" \
                         $i_height_dialog_progress \
                         $i_width_dialog_progress

    sleep 1
    if [ 1 -eq $i_fail_flag ];then
        s_abc="One or more tasks failed. Please check the configuration log for details.\n
  Please exit the wizard now, get the issues fixed and re-run the wizard by running below command:\n
    $0"
        if [ 1 -eq $i_status_ansible ];then
            s_abc="$s_abc \n  Installation of ansible python library has failed. Some of the filter expression   used in workflows might not work.\n
  This could be because of broken network connectivity to $product_yum_server.\n
  If the server connects via a proxy, ensure that the HTTPS_PROXY is set correctly   using the csadm network command. \n
  Once the network connectivity is verified, run the following command to reinstall   the library:\n
  sudo -u nginx /opt/cyops-workflow/.env/bin/pip install ansible --extra-index-url   https://$product_yum_server/prod/connectors/deps/simple/\n
  systemctl restart celeryd celerybeatd uwsgi"
            echo "$s_abc" >> $f_log
        fi
        sudo touch $f_vm_configuration_failed
        s_msg_config_finished=" $s_msg_full\n\n  $s_abc\n\n
  Configuration log is saved at $f_log."
        show_dialog_mbox "$s_title" \
                         "Exit" \
                         "$s_msg_config_finished" \
                         $i_height_dialog_progress \
                         $i_width_dialog_progress
        my_exit 1
    else
    if [ ! -f $f_hkey_save_file ];then
       # For Secure Message Exchange.
       s_cust_abc="$s_config_type is now configured.\n
  Note: You will be logged out now for changes to take effect.\n
  Please log in again."
  else
       # For FortiSOAR
       i_hkey=`sudo cat $f_hkey_save_file`
       s_cust_abc="$s_config_type is now configured.\n
  Device UUID: $i_hkey\n\n
  Please get license from support using the Device UUID displayed above.\n
  Refer documentation for details.\n
  Device UUID is saved in $f_hkey_save_file.\n\n
  Note: You will be logged out now for changes to take effect.\n
  Please log in again.\n"
    fi
        s_abc=$s_cust_abc
        s_msg_config_finished=" $s_msg_full\n\n  $s_cust_abc\n
  Configuration log is saved at \n
  $f_log."
        touch $f_vm_configured
        show_dialog_mbox "$s_title" \
                         "OK" \
                         "$s_msg_config_finished" \
                         $i_height_dialog_progress \
                         $i_width_dialog_progress
    fi
}

prepare_summary_message(){
    # Display proceed message on $i_proceed_msg_offset_y_axis th line
    #  from top of the dialog box
    i_proceed_msg_offset_y_axis=27

    # There are 5 lines from top till summary starts
    i_count_filler=`expr $i_proceed_msg_offset_y_axis - $i_count_summary_lines - 5`

    # Calculate number of empty lines between summary and proceed message
    s_empty_lines=$(printf '\\n%.0s' $(seq 1 $i_count_filler))

    s_display_summary="\nReady to configure $s_config_type.\n
\n
Please review the inputs before proceeding for configuration.\n
\n
${s_cust_summary}${s_empty_lines}\n
Press \"Proceed\" to start configuration.\nPress \"Back\" to startover."
}

show_summary(){
    i_height_dialog_summary=$i_height_dialog
    i_width_dialog_summary=$i_width_dialog

    prepare_summary_message
    s_summary=$s_display_summary

    show_dialog_yesno "$s_title" \
                      "Back" \
                      "Proceed" \
                      "$s_summary" \
                      $i_height_dialog_summary \
                      $i_width_dialog_summary
    return $?
}

is_valid_ipv6()
{
    local ipv6_addr=$1
    # following code will through exception if wrong IPv6 address. i.e return code is non-zero
    python3 -c "import ipaddress; ipaddress.IPv6Address('$ipv6_addr');" >/dev/null 2>&1
    if [ $? -eq 0 ]; then
        # 0 = true
        echo "0"
    else
        # 1 = false
        echo "1"
    fi
}

match_hostname_ip(){
    string="$1"
    type=$2

    if [ "hostname" = "$type" ];then
        [[ ! "$string" =~ $hostname_regex ]] && return 1 || return 0
    elif [ "both" = "$type" ];then
        # Try to match hostname first. If that fails try matching IP.
        # Return 1 if that fails
        valid_ipv6=`is_valid_ipv6 "$string"`
        [[ "$string" =~ $hostname_regex ]] && return 0 \
        || \
        [[ "$string" =~ $ip_regex || $valid_ipv6 =~ 0 ]] && return 0 \
        || return 1
    fi
}

get_dns(){
    i_height_dialog_get_dns=$i_height_dialog
    i_width_dialog_get_dns=$i_width_dialog
    while true; do
        show_dialog_inputbox "$s_title" \
                             "OK" \
                             "Skip" \
                             "\nEnter DNS server name or IP address [Optional]:\n\nNote: The DNS entries here are only for the initial configuration. \n
      After the instance is rebooted, the peer DNS identified by \n
      the Network Manager will be used." \
                             $i_height_dialog_get_dns \
                             $i_width_dialog_get_dns \
                             f_dns \
                             ""

        if [ $? -eq 0 ];then
            s_dns=`cat f_dns`
            if [ "X" != "X$s_dns" ];then
                match_hostname_ip "$s_dns" both
                if [ 0 -ne $? ];then
                    show_one_liner_mbox "\nInvalid hostname or IP. Please try again."
                    continue
                fi
                a_msgs=("${a_msgs[@]}" "Update network configuration")
                a_cmds=("${a_cmds[@]}" "sudo timeout $i_default_timeout_seconds csadm hostname --dns-name $s_dns")
                add_to_summary "DNS server" "$s_dns"
                break
            else
                add_to_summary "DNS server" "[Skipped]"
                break
            fi
        else
            add_to_summary "DNS server" "[Skipped]"
            break
        fi
    done
    rm -f f_dns
}

get_hostname(){
    s_hostname=`hostname`
    i_height_dialog_get_hostname=$i_height_dialog
    i_width_dialog_get_hostname=$i_width_dialog
    while true; do
        show_dialog_inputbox "$s_title" \
                             "OK" \
                             "Skip" \
                             "\nEnter new hostname [Optional]:" \
                             $i_height_dialog_get_hostname \
                             $i_width_dialog_get_hostname \
                             f_new_hostname \
                             "$s_hostname"
        if [ $? -eq 0 ];then
            s_new_hostname=`cat f_new_hostname`
            if [ "X$s_new_hostname" = "X" ];then
                show_one_liner_mbox "\nHostname cannot be empty."
            elif [ "$s_new_hostname" != "$s_hostname" ];then
                match_hostname_ip "$s_new_hostname" hostname
                if [ 0 -ne $? ];then
                    show_one_liner_mbox "\nInvalid hostname. Please try again."
                    continue
                fi
                s_hostname="$s_new_hostname"
                add_to_summary "Hostname" "$s_hostname"
                a_msgs=("${a_msgs[@]}" "Change hostname")
                # -z is for skip all confirmations
                a_cmds=("${a_cmds[@]}" "sudo timeout $i_default_timeout_seconds csadm hostname --set $s_hostname")
                export s_propogate_changes="true"
                break
            elif [ "$s_new_hostname" = "$s_hostname" ];then
                add_to_summary "Hostname" "[No Change]"
                a_msgs=("${a_msgs[@]}" "Set up messaging broker")
                a_cmds=("${a_cmds[@]}" "sudo timeout $i_default_timeout_seconds csadm mq --rabbitmq-hostname")
                break
            fi
        else
            add_to_summary "Hostname" "[No Change]"
            a_msgs=("${a_msgs[@]}" "Set up messaging broker")
            a_cmds=("${a_cmds[@]}" "sudo timeout $i_default_timeout_seconds csadm mq --rabbitmq-hostname")
            break
        fi
    done
    psql -U cyberpgsql sealab -c "insert into dynamic_variable_dynamicvariable (name, value) values ('Server_fqhn', '$s_new_hostname')"
    rm -f f_new_hostname
}

get_rabbitmq_credentials(){
    s_rb_user="admin"
    i_height_dialog_get_dns=$i_height_dialog
    i_width_dialog_get_dns=$i_width_dialog
    while true; do
        show_dialog_inputbox "$s_title" \
                             "OK" \
                             "Skip" \
                             "\nEnter Username for messaging configuration [Optional]:" \
                             $i_height_dialog_get_dns \
                             $i_width_dialog_get_dns \
                             f_rb_username \
                             "$s_rb_user"
         if [ $? -eq 0 ];then
            s_user=`cat f_rb_username`

            if [ "X" != "X$s_user" ];then
                 s_rb_user=$s_user
                 add_to_summary "Username" "$s_rb_user"
                 break;
            else
                add_to_summary "Username" "[Skipped]"
                break
            fi
         else
            add_to_summary "Username" "[Skipped]"
            break
         fi
    done


    i_height_dialog_get_new_db_pass=$i_height_dialog
    i_width_dialog_get_new_db_pass=$i_width_dialog
    rb_passwd_file=f_rb_passwd
    rb_conf_passwd_file=f_rb_conf_passwd
    while true; do
        # First attempt password
        show_dialog_passwdbox "$s_title" \
                             "OK" \
                             "Skip" \
                             "\nEnter password:" \
                             $i_height_dialog_get_new_db_pass \
                             $i_width_dialog_get_new_db_pass \
                             $rb_passwd_file \
                             ""
        # Check First attempt password empty or not
        s_rb_passwd=`cat $rb_passwd_file`
        if [ "X" = "X$s_rb_passwd" ]; then
            show_one_liner_mbox "\nPassword cannot be empty."
            continue
        elif [ ${#s_rb_passwd} -lt $i_password_length_mininum ] ; then
            # Password minimum length check
            show_one_liner_mbox "\nPassword must be of at least $i_password_length_mininum characters."
            continue
        fi

        # Confirm password
        show_dialog_passwdbox "$s_title" \
                             "OK" \
                             "Skip" \
                             "\nConfirm password:" \
                             $i_height_dialog_get_new_db_pass \
                             $i_width_dialog_get_new_db_pass \
                             $rb_conf_passwd_file \
                             ""
        if [ $? -eq 0 ];then
            # Check confirm password empty or not.
            s_rb_conf_passwd=`cat $rb_conf_passwd_file`
            if [ "X" != "X$s_rb_conf_passwd" ]; then
                # Check first attempt and confirm password same or not.
                if [ "$s_rb_passwd" = "$s_rb_conf_passwd" ];then
                    add_to_summary "Password" "********"
                    break
                else
                    show_one_liner_mbox "\nPassword doesn't match. Please enter password again."
                    continue
                fi
            else
                show_one_liner_mbox "\nPassword doesn't match. Please enter password again."
                continue
            fi
        else
            add_to_summary "Password" "[Skipped]"
            break
        fi
    done
    a_msgs=("${a_msgs[@]}" "Generate messaging credentials")
    a_cmds=("${a_cmds[@]}" "sudo timeout $i_default_timeout_seconds csadm mq --add-user $s_rb_user $s_rb_passwd")
    rm -f f_rb_passwd
    rm -f f_rb_conf_passwd
}

get_rabbitmq_vhost(){
  a_msgs=("${a_msgs[@]}" "Generate messaging virtual host")
  a_cmds=("${a_cmds[@]}" "sudo timeout $i_default_timeout_seconds csadm mq --add-vhost $s_rb_vhost")
  rm -f f_rb_username
}

enable_rabbitmq_mgt_console(){
   a_msgs=("${a_msgs[@]}" "Enable messaging management console")
   a_cmds=("${a_cmds[@]}" "sudo timeout $i_default_timeout_seconds csadm mq --enable-console")
}

enable_rabbitmq_event_listener_exchange(){
   a_msgs=("${a_msgs[@]}" "Enable messaging monitoring extension")
   a_cmds=("${a_cmds[@]}" "sudo timeout $i_default_timeout_seconds csadm mq --enable-event-plugin")
}

host_port_validation(){
      i_host_port=$1
     if [ -z "${i_host_port//[0-9]}" ] && [ -n "$i_host_port" ]
     then
        if [ $i_host_port -ge 49152 ] && [ $i_host_port -le 65535 ] || [ $i_host_port -eq 15671 ]
        then
           return 0
        else
           return 1
        fi
     else
        return 1
     fi

}

get_rabbitmq_port(){
    i_rb_port=15671
    i_height_dialog_get_dns=$i_height_dialog
    i_width_dialog_get_dns=$i_width_dialog
    while true; do
        show_dialog_inputbox "$s_title" \
                             "OK" \
                             "Skip" \
                             "\nEnter new port between 49152 to 65535 for messaging Management Console [Optional]: Default API Port: 15671" \
                             $i_height_dialog_get_dns \
                             $i_width_dialog_get_dns \
                             f_rb_port \
                             "$i_rb_port"

        if [ $? -eq 0 ];then
            i_rb_port=`cat f_rb_port`
            if [ "X" != "X$i_rb_port" ];then
                host_port_validation $i_rb_port
                if [ $? -ne 0 ]; then
                   show_one_liner_mbox "The port number should be interger and between 49152 to 65535."
                   continue
                else
                  a_msgs=("${a_msgs[@]}" "Set new port for messaging")
                  a_cmds=("${a_cmds[@]}" "sudo timeout $i_default_timeout_seconds csadm mq --tcp-rabbitmq-port $i_rb_port")
                  add_to_summary "API Port" "$i_rb_port"
                  break
                fi
            else
                add_to_summary "API Port" "[Skipped]"
                break
            fi
        else
            add_to_summary "API Port" "[Skipped]"
            break
        fi
    done
    rm -f f_rb_port
}

get_rabbitmq_tcp_port(){
    i_rb_tcp_port=5671
    i_height_dialog_get_dns=$i_height_dialog
    i_width_dialog_get_dns=$i_width_dialog
    while true; do
        show_dialog_inputbox "$s_title" \
                             "OK" \
                             "Skip" \
                             "\nEnter new port between 49152 to 65535 for messaging TCP [Optional]: Default TCP Port: 5671" \
                             $i_height_dialog_get_dns \
                             $i_width_dialog_get_dns \
                             f_rb_tcp_port \
                             "$i_rb_tcp_port"

        if [ $? -eq 0 ];then
            i_rb_tcp_port=`cat f_rb_tcp_port`
            if [ "X" != "X$i_rb_tcp_port" ];then
                host_port_validation $i_rb_tcp_port
                if [ $? -ne 0 ]; then
                   if [ $i_rb_tcp_port -ne 5671 ]; then
                     show_one_liner_mbox "The port number should be interger and between 49152 to 65535."
                     continue
                   else
                     a_msgs=("${a_msgs[@]}" "Set new tcp port for messaging")
                     a_cmds=("${a_cmds[@]}" "sudo timeout $i_default_timeout_seconds csadm mq --new-tcp-rabbitmq-port $i_rb_tcp_port")
                     add_to_summary "TCP Port" "$i_rb_tcp_port"
                     break
                   fi
                else
                  if [ $i_rb_port != $i_rb_tcp_port  ]; then
                     a_msgs=("${a_msgs[@]}" "Set new tcp port for messaging")
                     a_cmds=("${a_cmds[@]}" "sudo timeout $i_default_timeout_seconds csadm mq --new-tcp-rabbitmq-port $i_rb_tcp_port")
                     add_to_summary "TCP Port" "$i_rb_tcp_port"
                     break
                  else
                     show_one_liner_mbox "The messaging API and TCP ports should not be same. Enter different port for TCP."
                     continue
                  fi
                fi
            else
                add_to_summary "TCP Port" "[Skipped]"
                break
            fi
        else
            add_to_summary "TCP Port" "[Skipped]"
            break
        fi
    done
    rm -f f_rb_tcp_port
}

delete_rabbitmq_user(){
  a_msgs=("${a_msgs[@]}" "Delete messaging default user")
  a_cmds=("${a_cmds[@]}" "sudo timeout $i_default_timeout_seconds csadm mq --delete-user $s_rb_default_user")
}

rabbitmq_env(){
  a_msgs=("${a_msgs[@]}" "Enable messaging environment")
  a_cmds=("${a_cmds[@]}" "sudo timeout $i_default_timeout_seconds csadm mq --enable-rabbitmq-env")
}

# Insert or replace the data in the file $f_ifcfg
insert_or_replace(){
    s_key=$1
    s_value=$2

    if [ -z "$s_value" ];then
        # Optional entry
        return 0
    fi

    grep $s_key $f_ifcfg > /dev/null
    if [ $? -eq 0 ];then
        sed -i "s/$s_key=\(.*\)/$s_key=$s_value/g" $f_ifcfg
    else
        echo "$s_key=$s_value" >> $f_ifcfg
    fi
}

update_invalid_ip_list(){
    s_address="$1"
    i_count_invalid_ip=`expr $i_count_invalid_ip + 1`
    if [ -z "$s_address" ];then
        l_invalid_address=" - $s_address\n"
    else
        l_invalid_address="$l_invalid_address - $s_address\n"
    fi
}

validate_ip(){
    s_type=$1
    s_display="$2"
    s_ip="$3"

    # If its optional and empty, return 0
    # Ex: DNS2
    if [ "$s_type" = "optional" -a -z "$s_ip" ];then
        return 0
    fi

    if [ -z "$s_ip" ];then
        update_invalid_ip_list "$s_display"
        flag_blank_entry=1
        # Empty IP field
        return 1
    fi

    if [[ ! "$s_ip" =~ $ip_regex ]];then
        update_invalid_ip_list "$s_display"
        return 1
    fi
}

validate_input(){
    # Validate the IPs
    flag_blank_entry=0
    i_count_invalid_ip=0
    l_invalid_address=""

    validate_ip mandatory "IP address" $s_static_ip

    validate_ip mandatory "Gateway address" $s_gateway

    validate_ip mandatory "Netmask address" $s_netmask

    validate_ip mandatory "DNS1 address" $s_dns1

    validate_ip optional "DNS2 address" $s_dns2

    if [ $i_count_invalid_ip -gt 0 ];then
        # Show additional line in case any of the mandatory fields are left blank
        s_msg_blank=""
        i_blank_line_count=0
        if [ $flag_blank_entry -eq 1 ];then
            s_msg_blank="\n(Fields marked by * cannot be left blank)"
            i_blank_line_count=1
        fi
        s_msg="\nInvalid data in below listed fields:$s_msg_blank\n$l_invalid_address\nPlease try again."
        i_dialog_height=`expr $i_count_invalid_ip + 8 + $i_blank_line_count`
        i_dialog_width=50
        show_dialog_mbox "$s_title" \
                         "OK" \
                         "$s_msg" \
                         $i_dialog_height \
                         $i_dialog_width
        return 1
    else
        return 0
    fi
}

add_to_summary(){
    s_label="$1"
    s_value="$2"
    i_length_label=${#s_label}

    # X co-ordinate on screen at which to display the value
    i_value_display_offset_x_axis=30
    i_count_filler=`expr $i_value_display_offset_x_axis - $i_length_label - 2`
    s_offset=$(printf -- '-%.0s' $(seq 1 $i_count_filler))
    s_cust_summary="$s_cust_summary${s_label} ${s_offset} : $s_value\n"
    i_count_summary_lines=`expr $i_count_summary_lines + 1`
}

update_ifcfg(){
    # Insert if not found, replace otherwise
    insert_or_replace IPADDR $s_static_ip
    add_to_summary "Static IP" "$s_static_ip"

    insert_or_replace GATEWAY $s_gateway
    add_to_summary "Gateway" "$s_gateway"

    insert_or_replace NETMASK $s_netmask
    add_to_summary "Netmask" "$s_netmask"

    insert_or_replace DNS1 $s_dns1
    add_to_summary "DNS1" "$s_dns1"

    insert_or_replace DNS2 $s_dns2
    if [ ! -z "$s_dns2" ];then
        add_to_summary "DNS2" "$s_dns2"
    else
        add_to_summary "DNS2" "[Empty]"
    fi

    insert_or_replace BOOTPROTO static
}

get_static_ip(){
    s_dialog_form_title="Set static IP"
    s_dialog_form_label_input1="Static IP* :"
    s_dialog_form_label_input2="Gateway*   :"
    s_dialog_form_label_input3="Netmask*   :"
    s_dialog_form_label_input4="DNS1*      :"
    s_dialog_form_label_input5="DNS2       :"
    f_dialog_form_output="output.txt"
    s_dialog_form_text="\nPlease enter static IP details:\n(Fields marked with * are mandatory)"
    while true; do
        show_dialog_form_five   "$s_dialog_form_title"  \
                                "$s_dialog_form_label_input1" \
                                "$s_dialog_form_label_input2" \
                                "$s_dialog_form_label_input3" \
                                "$s_dialog_form_label_input4" \
                                "$s_dialog_form_label_input5" \
                                "$f_dialog_form_output" \
                                "$s_dialog_form_text"
        s_static_ip=$(cat $f_dialog_form_output | sed -n '1p' )
        s_gateway=$(cat $f_dialog_form_output | sed -n '2p' )
        s_netmask=$(cat $f_dialog_form_output | sed -n '3p' )
        s_dns1=$(cat $f_dialog_form_output | sed -n '4p' )
        s_dns2=$(cat $f_dialog_form_output | sed -n '5p')
        rm -f $f_dialog_form_output
        validate_input
        if [ $? -ne 0 ];then
            s_dialog_form_value_input1=$s_static_ip
            s_dialog_form_value_input2=$s_gateway
            s_dialog_form_value_input3=$s_netmask
            s_dialog_form_value_input4=$s_dns1
            s_dialog_form_value_input5=$s_dns2
            continue
        fi

        update_ifcfg
        systemctl restart network
        break
    done
}

update_mac_address(){
    # Replace the MAC address
    # Why: The MAC address ifcfg file currently has is of the VM from which the VAPP was created
    # Once the VAPP is imported, it gets new MAC. Need to replace this new MAC
    s_mac_address=`cat /sys/class/net/$s_if/address`

    insert_or_replace HWADDR $s_mac_address
}

get_inputs(){
    get_hostname

    # VM has old mac address in ifcfg created at the time of VM generation
    # Once VM is imported, it gets new MAC
    # So update the ifcfg with new MAC adderss
    update_mac_address

    if [ $flag_set_static_ip -eq 1 ];then
        get_static_ip
    else
        get_dns
    fi
}

reset_variables(){
    s_summary=""
    s_cust_summary=""
    i_count_summary_lines=0
    a_msgs=()
    a_cmds=()
}

show_welcome_box(){
    i_height_welcome_box=$i_height_dialog
    i_width_welcome_box=$i_width_dialog
    s_title_welcome_box=$s_cust_title_welcome_box
    s_msg_welcome_box=$s_msg_cust_welcome_box
    show_dialog_mbox "$s_title_welcome_box" \
                     "OK" \
                     "$s_msg_welcome_box" \
                     $i_height_welcome_box \
                     $i_width_welcome_box
}



check_pid(){
    i_pid=$1
    ps -ef | grep -w $i_pid | grep -v grep > /dev/null 2>&1
    if [ 0 -eq $? ];then
        return 0
    else
        return 1
    fi
}

show_dialog_wait_for_config_to_finish(){
    s_msg="$1"
    i_height_dialog_wait=3
    s_title=" Please Wait "
    i_msg_len=${#s_msg}
    i_width_dialog_wait=`expr $i_msg_len + 7`
    s_msg_show="$s_msg $i"
    show_dialog_infobox "$s_title" \
                        "$s_msg" \
                        $i_height_dialog_wait \
                        $i_width_dialog_wait
}

wait_for_config_to_finish(){

    while true; do
        i_wait_seconds=`expr $i_wait_minutes \* $i_seconds_per_minute`
        i_sleep=1
        while [ 0 -lt $i_wait_seconds ]; do
            if [ $i_wait_seconds -lt $i_seconds_per_minute ];then
                i_display_time="$i_wait_seconds seconds"
            else
                i_display_time_minutes=`expr $i_wait_seconds / $i_seconds_per_minute`
                i_display_time_seconds=`expr $i_wait_seconds % $i_seconds_per_minute`
                i_display_time="$i_display_time_minutes:$i_display_time_seconds minutes"
            fi
            show_dialog_wait_for_config_to_finish "FortiSOAR configuration is in progress in another session. Waiting for $i_display_time to finish"
            sleep $i_sleep
            i_wait_seconds=`expr $i_wait_seconds - $i_sleep`
            check_pid $i_pid_vm_config
            if [ 1 -eq $? ];then
                if [ ! -f "$f_vm_configured" ];then
                    # pid not found and .vm_configured also not found
                    # that means last session was timed out
                    # so configuration is not running
                    # So re-start configuration
                    show_config_timeout_mbox
                    return 0
                else
                    # pid not found and .vm_configured found
                    # Configuration has been successfully completed
                    # Login to command prompt
                    my_exit 0
                fi
            else
                # pid found
                continue
            fi
        done
        check_pid $i_pid_vm_config
        if [ 1 -eq $? ];then
            if [ ! -f "$f_vm_configured" ];then
                # pid not found and .vm_configured also not found
                # that means last session was timed out
                # so configuration is not running
                # So re-start configuration
                show_config_timeout_mbox
                return 0
            else
                # pid not found and .vm_configured found
                # Configuration has been successfully completed
                # Login to command prompt
                my_exit 0
            fi
        else
            # ask user to wait more or restart configuration
            s_msg="Looks like configuration is still running.\n
You have two options:\n
  1. Wait for $i_wait_minutes more minutes\n
  2. Restart the configuration [This will kill currently running configuration and start new one]"
            show_dialog_yesno "$s_title" \
                              "Wait for $i_wait_minutes more minutes" \
                              "Restart the configuration" \
                              "$s_msg" \
                              $i_height_dialog \
                              $i_width_dialog
            if [ 0 -eq $? ];then
                # Wait for $i_wait_minutes more
                continue
            else
                # Kill currently running config and restart the config
                kill -9 $i_pid_vm_config > /dev/null 2>&1
                return 0
            fi
        fi
    done
}

show_config_timeout_mbox(){
    show_one_liner_mbox "FortiSOAR configuration wizard was timed out in another session. Restarting FortiSOAR configuration."
}

check_prev_failure(){
    if [ -f $f_vm_configuration_failed ];then
         s_warning_msg="\n\nFortiSOAR configuration failed in last run.\n
\n\n\n\n\n\n\n\n
Press \"Proceed\" to configure FortiSOAR, if issues are fixed.\n
Press \"Exit\" to exit the wizard."
         show_dialog_yesno "$s_title" \
                           "Exit" \
                           "Proceed" \
                           "$s_warning_msg" \
                           $i_height_dialog \
                           $i_width_dialog
        if [ 0 -eq $? ];then
            my_exit 0
        fi
        rm -f $f_vm_configuration_failed
        return 0
    else
        return 2
    fi
}

check_for_another_vm_config_session(){
    check_prev_failure
    if [ 0 -eq $? ];then
        # If user chooses to reconfigure, skip further checks
        return 0
    fi

    if [ ! -f "$f_vm_configured" ];then

        if [ -f "$f_pid_vm_config" ];then
            # pid file found
            # Double check to see if the pid in the file too exists
            i_pid_vm_config=`cat $f_pid_vm_config`
            check_pid $i_pid_vm_config
            if [ 0 -eq $? ];then
                # .vm_configured file not found
                # pid file found and valid pid too is found
                # Configuration is really running
                wait_for_config_to_finish
                return $?
            else
                # .vm_configured file not found
                # pid file found but valid pid not found
                # that means last session was timed out
                # so configuration is not running
                # So re-start configuration
                show_config_timeout_mbox
                return 0
            fi
        else
            # pid file not found and .vm_configured file not found
            # So good to start with configuration
            return 0
        fi
    else
        # .vm_configured file found
        # Configuration has been successfully completed
        # Login to command prompt
        my_exit 0
    fi
}

init(){
    # Disable control C
    trap "" SIGINT

    # Fix for garbeled chars showing up in putty
    export NCURSES_NO_UTF8_ACS=1

    i_height_dialog=35
    i_width_dialog=90

    i_height_form=20
    i_width_form=60

    i_height_inputbox=13
    i_width_inputbox=60

    i_cols=55

    i_spinner_delay=0.5

    i_count_summary_lines=0

    f_script_name=`basename $0`
    f_pid_vm_config=/home/csadmin/${f_script_name}.pid
    f_script_name="${f_script_name%.*}"

    # Removes pid file upon exit
    trap "rm -f -- '$f_pid_vm_config'" EXIT

    i_wait_minutes=10
    i_seconds_per_minute=60
    i_default_timeout_seconds=900

    i_password_length_mininum=8

    i_hkey=""

    # Rabbitmq Parameters
    s_rb_default_user="guest"
    s_rb_vhost="intra-cyops"
    i_rb_default_port=15671

    f_loopbackp="/etc/rabbitmq/rabbitmq.conf"

    f_dialog_cmd=dialog

    s_cyops_name_secure_message_exchange="secure-message-exchange"
    s_cyops_name_enterprise="enterprise"
    s_cyops_name_securonix="securonix"
    f_cyops_os_release_file="/etc/cyops-release"
    d_pgsql_path="/var/lib/pgsql"
    f_pgsql_resv_space_file="resv_space"

    f_config_script="/opt/cyops/configs/scripts/regenerate_default_configs.sh"

    f_hkey_save_file="/home/csadmin/device_uuid"
    f_vm_configured=/home/csadmin/.vm_configured
    f_vm_configuration_failed=/home/csadmin/.vm_configuration_failed

    f_bash_profile="/home/csadmin/.bash_profile"
    s_my_date=`date +%d_%b_%Y_%Hh_%Mm_%Ss`
    f_log=/var/log/cyops/install/config-vm-${s_my_date}.log

    # Initializations
    s_backtitle="FortiSOAR Configuration"
    s_config_type="FortiSOAR"
    s_title="FortiSOAR Configuration"
    check_for_another_vm_config_session

    hostname_regex="^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\-]*[A-Za-z0-9])$"
    ip_regex="^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$"
    s_if=ens160
    f_ifcfg="/etc/sysconfig/network-scripts/ifcfg-$s_if"

    echo $$ > "$f_pid_vm_config"

    # Check if IP is assigned to this VM
    # If yes, set the flag and step label accordingly
    flag_set_static_ip=0
    s_step_two="2. Get DNS"
    s_cur_ip=`hostname -I`
    if [ -z "$s_cur_ip" ];then
        flag_set_static_ip=1
        s_step_two="2. Set Static IP"
    fi
}

########
# MAIN #
########
init

if [ ! -f $f_vm_configured ];then

     s_rb_vhost="cyops-admin"
     s_cyops_os_release=`grep "product_type" $f_cyops_os_release_file | cut -d'=' -f2`

     if [ "$s_cyops_name_secure_message_exchange" = "$s_cyops_os_release" ]; then
         s_backtitle="FortiSOAR Secure Message Exchange Configuration"
         s_title="FortiSOAR Secure Message Exchange Configuration"
         s_config_type="FortiSOAR Secure Message Exchange"
         s_cust_title="Configuring $s_config_type"
         s_cust_backtitle="FortiSOAR Secure Message Exchange Configuration"
         s_backtitle=$s_cust_backtitle
         s_cust_title_welcome_box="Welcome to FortiSOAR Secure Message Exchange Configuration Wizard"

         f_config_cust_script="/opt/cyops-rabbitmq/configs/ssl/rabbitmq_certs.sh"

         s_msg_cust_welcome_box="\n
This wizard helps you to perform one-time FortiSOAR Secure Message Exchange configuration.\n
Following tasks will be carried out as part of configuration:\n
    1. Change hostname [Optional]\n
    $s_step_two\n
    3. Enable messaging environment\n
    4. Delete messaging default user\n
    5. Generate messaging credentials\n
    6. Set new port for messaging\n
    7. Set new tcp port for messaging\n
    8. Generate messaging virtual host\n
    9. Enable messaging management console\n
    10. Generate certificates\n

\n\n\n\n\nPress \"OK\" to proceed."

         show_welcome_box
         while true; do
            reset_variables

            get_inputs
            rabbitmq_env
            delete_rabbitmq_user
            get_rabbitmq_credentials
            get_rabbitmq_port
            get_rabbitmq_tcp_port
            get_rabbitmq_vhost
            enable_rabbitmq_mgt_console
            enable_rabbitmq_event_listener_exchange
            #Delete the loopbackup user.
            sed -i "/loopback_users/d" $f_loopbackp
            show_summary
            if [ 1 -eq $? ];then
              break
            fi
        done

        add_default_rabbitmq_steps

        configure_vm

        my_exit 0

    elif [ "$s_cyops_name_enterprise" = "$s_cyops_os_release" ] || \
         [ "$s_cyops_name_securonix" = "$s_cyops_os_release" ]; then
         s_cust_title_welcome_box="Welcome to FortiSOAR Configuration Wizard"
         s_cust_title="Configuring $s_config_type"

         s_msg_cust_welcome_box="\n
 This wizard helps you to perform one-time FortiSOAR configuration.\n\n
 Following tasks will be carried out as part of configuration:\n
     1. Change hostname [Optional]\n
     $s_step_two\n
     3. Set up messaging broker [Optional]\n
     4. Update network configuration [Optional]\n
     5. Set up intra-service authentication\n
     6. Generate certificates\n
     7. Generate Device UUID\n
     8. Reset database passwords\n
     9. Restart services\n
    10. Configure default HA cluster\n
    11. Configure proxy\n
    12. Install python libraries\n
 \n\n\n\n\n\n\nPress \"OK\" to proceed."

 show_welcome_box
        while true; do
           reset_variables

           get_inputs
           set_proxy
           show_summary
           if [ 1 -eq $? ];then
             break
           fi
        done

        add_default_steps

        configure_vm

        # Reserve the 1 GB space, in case disk full issue the user,
        # will delete the file /var/lib/pgsql/resv_space and utilize the space
        # when require and create the file again using the following command once issue resolve.
        # The following command is dd if=/dev/zero of=/var/lib/pgsql/resv_space count=1024 bs=1048576
        dd if=/dev/zero of=$d_pgsql_path/$f_pgsql_resv_space_file count=1024 bs=1048576
        if [ $? -ne 0 ];then
            echo "Failed to create the $d_pgsql_path/$f_pgsql_resv_space_file file. Make sure $d_pgsql_path has sufficient space. Please re-run the following command manually.
dd if=/dev/zero of=$d_pgsql_path/$f_pgsql_resv_space_file count=1024 bs=1048576"
        fi

        my_exit 0

     fi
elif [ -f $f_vm_configured ];then

    show_one_liner_mbox "FortiSOAR is already configured. Exiting configuration wizard."
    my_exit 0
fi
